const http = require('http');
const { spawn } = require('child_process');

const server = http.createServer((req, res) => {
  if (req.method === 'GET' && req.url === '/run-tests') {
    // 使用 --inspect-brk 标志生成 Jest 进程
    const jestProcess = spawn('npm', ['test', '--inspect-brk'], { cwd: __dirname });

    jestProcess.stdout.on('data', data => {
      console.log(data.toString());
    });

    jestProcess.stderr.on('data', data => {
      console.error(data.toString());
    });

    jestProcess.on('exit', code => {
      if (code === 0) {
        res.writeHead(200, { 'Content-Type': 'text/plain' });
        res.end('测试已成功执行。');
      } else {
        res.writeHead(500, { 'Content-Type': 'text/plain' });
        res.end('测试执行失败。');
      }
    });
  } else {
    res.writeHead(404, { 'Content-Type': 'text/plain' });
    res.end('未找到。');
  }
});

const PORT = 8080;
server.listen(PORT, () => {
  console.log(`服务器正在 http://localhost:${PORT} 上运行。`);
});
